Class: Commander::Runner

Inherits:
Object show all
Defined in:
lib/simple_commander/runner.rb

Defined Under Namespace

Classes: CommandError, InvalidCommandError

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(args = ARGV) ⇒ Runner

Initialize a new command runner. Optionally supplying args for mocking, or arbitrary usage.



31
32
33
34
35
36
37
38
# File 'lib/simple_commander/runner.rb', line 31

def initialize(args = ARGV)
  @args, @commands, @aliases, @options = args, {}, {}, []
  @help_formatter_aliases = help_formatter_alias_defaults
  @program = program_defaults
  @always_trace = false
  @never_trace = false
  create_default_commands
end

Instance Attribute Details

#commandsObject (readonly)

Array of commands.



15
16
17
# File 'lib/simple_commander/runner.rb', line 15

def commands
  @commands
end

#help_formatter_aliasesObject (readonly)

Hash of help formatter aliases.



25
26
27
# File 'lib/simple_commander/runner.rb', line 25

def help_formatter_aliases
  @help_formatter_aliases
end

#optionsObject (readonly)

Global options.



20
21
22
# File 'lib/simple_commander/runner.rb', line 20

def options
  @options
end

Class Method Details

.instanceObject

Return singleton Runner instance.



43
44
45
# File 'lib/simple_commander/runner.rb', line 43

def self.instance
  @singleton ||= new
end

.separate_switches_from_description(*args) ⇒ Object

Return switches and description separated from the args passed.



439
440
441
442
443
# File 'lib/simple_commander/runner.rb', line 439

def self.separate_switches_from_description(*args)
  switches = args.find_all { |arg| arg.to_s =~ /^-/ }
  description = args.last if args.last.is_a?(String) && !args.last.match(/^-/)
  [switches, description]
end

.switch_to_sym(switch) ⇒ Object

Attempts to generate a method name symbol from switch. For example:

-h                 # => :h
--trace            # => :trace
--some-switch      # => :some_switch
--[no-]feature     # => :feature
--file FILE        # => :file
--list of,things   # => :list


457
458
459
# File 'lib/simple_commander/runner.rb', line 457

def self.switch_to_sym(switch)
  switch.scan(/[\-\]](\w+)/).join('_').to_sym rescue nil
end

Instance Method Details

#active_commandObject

Get active command within arguments passed to this runner.



257
258
259
# File 'lib/simple_commander/runner.rb', line 257

def active_command
  @__active_command ||= command(command_name_from_args)
end

#add_command(command) ⇒ Object

Add a command object to this runner.



234
235
236
# File 'lib/simple_commander/runner.rb', line 234

def add_command(command)
  @commands[command.name] = command
end

#alias?(name) ⇒ Boolean

Check if command name is an alias.

Returns:

  • (Boolean)


241
242
243
# File 'lib/simple_commander/runner.rb', line 241

def alias?(name)
  @aliases.include? name.to_s
end

#alias_command(alias_name, name, *args) ⇒ Object

Alias command name with alias_name. Optionally args may be passed as if they were being passed straight to the original command via the command-line.



218
219
220
221
# File 'lib/simple_commander/runner.rb', line 218

def alias_command(alias_name, name, *args)
  @commands[alias_name.to_s] = command name
  @aliases[alias_name.to_s] = args
end

#always_trace!Object

Enable tracing on all executions (bypasses –trace)



118
119
120
121
# File 'lib/simple_commander/runner.rb', line 118

def always_trace!
  @always_trace = true
  @never_trace = false
end

#args_without_command_nameObject

Return arguments without the command name.



287
288
289
290
291
292
293
# File 'lib/simple_commander/runner.rb', line 287

def args_without_command_name
  removed = []
  parts = command_name_from_args.split rescue []
  @args.dup.delete_if do |arg|
    removed << arg if parts.include?(arg) && !removed.include?(arg)
  end
end

#command(name, &block) ⇒ Object

Creates and yields a command instance when a block is passed. Otherwise attempts to return the command, raising InvalidCommandError when it does not exist.

Examples

command :my_command do |c|
  c.when_called do |args|
    # Code
  end
end


189
190
191
192
193
194
195
196
197
198
# File 'lib/simple_commander/runner.rb', line 189

def command(name, &block)
			Commander::Command.new(name).tap do |cmd|
add_command(cmd) if block
cmd.super_self = self
cmd.instance_eval &block if block
			end
			#add_command(Commander::Command.new(name)) if block
  #yield add_command(Commander::Command.new(name)) if block
  @commands[name.to_s]
end

#command_exists?(name) ⇒ Boolean

Check if a command name exists.

Returns:

  • (Boolean)


248
249
250
# File 'lib/simple_commander/runner.rb', line 248

def command_exists?(name)
  @commands[name.to_s]
end

#command_name_from_argsObject

Attempts to locate a command name from within the arguments. Supports multi-word commands, using the largest possible match.



265
266
267
# File 'lib/simple_commander/runner.rb', line 265

def command_name_from_args
  @__command_name_from_args ||= (valid_command_names_from(*@args.dup).sort.last || @default_command)
end

#create_default_commandsObject

Creates default commands such as ‘help’ which is essentially the same as using the –help switch.



318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
# File 'lib/simple_commander/runner.rb', line 318

def create_default_commands
			command :help do
    syntax = 'commander help [command]'
    description = 'Display global or [command] help documentation'
    example 'Display global help', 'command help'
    example "Display help for 'foo'", 'command help foo'
    when_called do |args, _options|
      UI.enable_paging
      if args.empty?
        say help_formatter.render
      else
        command = command args.join(' ')
        begin
          require_valid_command command
        rescue InvalidCommandError => e
          abort "#{e}. Use --help for more information"
        end
        say help_formatter.render_command(command)
      end
    end
  end
end

#default_command(name) ⇒ Object

Default command name to be used when no other command is found in the arguments.



227
228
229
# File 'lib/simple_commander/runner.rb', line 227

def default_command(name)
  @default_command = name
end

#expand_optionally_negative_switches(switches) ⇒ Object

expand switches of the style ‘–[no-]blah’ into both their ‘–blah’ and ‘–no-blah’ variants, so that they can be properly detected and removed



383
384
385
386
387
388
389
390
391
392
# File 'lib/simple_commander/runner.rb', line 383

def expand_optionally_negative_switches(switches)
  switches.reduce([]) do |memo, val|
    if val =~ /\[no-\]/
      memo << val.gsub(/\[no-\]/, '')
      memo << val.gsub(/\[no-\]/, 'no-')
    else
      memo << val
    end
  end
end

#global_option(*args, &block) ⇒ Object

Add a global option; follows the same syntax as Command#option This would be used for switches such as –version, –trace, etc.



204
205
206
207
208
209
210
211
212
# File 'lib/simple_commander/runner.rb', line 204

def global_option(*args, &block)
  switches, description = Runner.separate_switches_from_description(*args)
  @options << {
    args: args,
    proc: block,
    switches: switches,
    description: description,
  }
end

#global_option_proc(switches, &block) ⇒ Object

Returns a proc allowing for commands to inherit global options. This functionality works whether a block is present for the global option or not, so simple switches such as –verbose can be used without a block, and used throughout all commands.



418
419
420
421
422
423
424
425
# File 'lib/simple_commander/runner.rb', line 418

def global_option_proc(switches, &block)
  lambda do |value|
    unless active_command.nil?
      active_command.proxy_options << [Runner.switch_to_sym(switches.last), value]
    end
    yield value if block && !value.nil?
  end
end

#goto_child_commandObject

make the child command the active_command and remove the current command from the @args



101
102
103
104
105
106
# File 'lib/simple_commander/runner.rb', line 101

def goto_child_command 
	@args.shift
	@__active_command = nil
	@__command_name_from_args = nil
	have_action?
end

#have_action?Boolean

tests if the current active command have an action block

Returns:

  • (Boolean)


93
94
95
# File 'lib/simple_commander/runner.rb', line 93

def have_action?
	goto_child_command if active_command.has_no_action?
end

#help_formatterObject

Help formatter instance.



280
281
282
# File 'lib/simple_commander/runner.rb', line 280

def help_formatter
  @__help_formatter ||= program(:help_formatter).new self
end

#help_formatter_alias_defaultsObject

Returns hash of help formatter alias defaults.



298
299
300
301
302
# File 'lib/simple_commander/runner.rb', line 298

def help_formatter_alias_defaults
  {
    compact: HelpFormatter::TerminalCompact,
  }
end

#never_trace!Object

Hide the trace option from the help menus and don’t add it as a global option



126
127
128
129
# File 'lib/simple_commander/runner.rb', line 126

def never_trace!
  @never_trace = true
  @always_trace = false
end

#parse_global_optionsObject

Parse global command options.



397
398
399
400
401
402
403
404
405
406
407
408
409
410
# File 'lib/simple_commander/runner.rb', line 397

def parse_global_options
  parser = options.inject(OptionParser.new) do |options, option|
    options.on(*option[:args], &global_option_proc(option[:switches], &option[:proc]))
  end

  options = @args.dup
  begin
    parser.parse!(options)
  rescue OptionParser::InvalidOption => e
    # Remove the offending args and retry.
    options = options.reject { |o| e.args.include?(o) }
    retry
  end
end

#program(key, *args, &block) ⇒ Object

Assign program information.

Examples

# Set data
program :name, 'Commander'
program :version, Commander::VERSION
program :description, 'Commander utility program.'
program :help, 'Copyright', '2008 TJ Holowaychuk'
program :help, 'Anything', 'You want'
program :int_message 'Bye bye!'
program :help_formatter, :compact
program :help_formatter, Commander::HelpFormatter::TerminalCompact

# Get data
program :name # => 'Commander'

Keys

:version         (required) Program version triple, ex: '0.0.1'
:description     (required) Program description
:name            Program name, defaults to basename of executable
:help_formatter  Defaults to Commander::HelpFormatter::Terminal
:help            Allows addition of arbitrary global help blocks
:int_message     Message to display when interrupted (CTRL + C)


159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
# File 'lib/simple_commander/runner.rb', line 159

def program(key, *args, &block)
  if key == :help && !args.empty?
    @program[:help] ||= {}
    @program[:help][args.first] = args.at(1)
  elsif key == :help_formatter && !args.empty?
    @program[key] = (@help_formatter_aliases[args.first] || args.first)
  elsif block
    @program[key] = block
  else
    unless args.empty?
      @program[key] = (args.count == 1 && args[0]) || args
    end
    @program[key]
  end
end

#program_defaultsObject

Returns hash of program defaults.



307
308
309
310
311
312
# File 'lib/simple_commander/runner.rb', line 307

def program_defaults
  {
    help_formatter: HelpFormatter::Terminal,
    name: File.basename($PROGRAM_NAME),
  }
end

#remove_global_options(options, args) ⇒ Object

Removes global options from args. This prevents an invalid option error from occurring when options are parsed again for the command.



353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
# File 'lib/simple_commander/runner.rb', line 353

def remove_global_options(options, args)
  # TODO: refactor with flipflop, please TJ ! have time to refactor me !
  options.each do |option|
    switches = option[:switches].dup
    next if switches.empty?

    if (switch_has_arg = switches.any? { |s| s =~ /[ =]/ })
      switches.map! { |s| s[0, s.index('=') || s.index(' ') || s.length] }
    end

    switches = expand_optionally_negative_switches(switches)

    past_switch, arg_removed = false, false
    args.delete_if do |arg|
      if switches.any? { |s| s[0, arg.length] == arg }
        arg_removed = !switch_has_arg
        past_switch = true
      elsif past_switch && !arg_removed && arg !~ /^-/
        arg_removed = true
      else
        arg_removed = true
        false
      end
    end
  end
end

#require_program(*keys) ⇒ Object

Raises a CommandError when the program any of the keys are not present, or empty.



430
431
432
433
434
# File 'lib/simple_commander/runner.rb', line 430

def require_program(*keys)
  keys.each do |key|
    fail CommandError, "program #{key} required" if program(key).nil? || program(key).empty?
  end
end

#require_valid_command(command = active_command) ⇒ Object

Raises InvalidCommandError when a command is not found.



344
345
346
# File 'lib/simple_commander/runner.rb', line 344

def require_valid_command(command = active_command)
  fail InvalidCommandError, 'invalid command', caller if command.nil?
end

#run!Object

Run command parsing and execution process.



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
# File 'lib/simple_commander/runner.rb', line 50

def run!
			trace = @always_trace || false
  require_program :version, :description
  trap('INT') { abort program(:int_message) } if program(:int_message)
  trap('INT') { program(:int_block).call } if program(:int_block)
  global_option('-h', '--help', 'Display help documentation') do
    args = @args - %w(-h --help)
    command(:help).run(*args)
    return
  end
  global_option('-v', '--version', 'Display version information') do
    say version
    return
  end
  global_option('-t', '--trace', 'Display backtrace when an error occurs') { trace = true } unless @never_trace || @always_trace
  parse_global_options
  remove_global_options options, @args
			have_action?
  if trace
    run_active_command
  else
    begin
      run_active_command
    rescue InvalidCommandError => e
      abort "#{e}. Use --help for more information"
    rescue \
      OptionParser::InvalidOption,
      OptionParser::InvalidArgument,
      OptionParser::MissingArgument => e
      abort e.to_s
    rescue => e
      if @never_trace
        abort "error: #{e}."
      else
        abort "error: #{e}. Use --trace to view backtrace"
      end
    end
  end
end

#run_active_commandObject

Run the active command.



464
465
466
467
468
469
470
471
# File 'lib/simple_commander/runner.rb', line 464

def run_active_command
  require_valid_command
  if alias? command_name_from_args
    active_command.run(*(@aliases[command_name_from_args.to_s] + args_without_command_name))
  else
    active_command.run(*args_without_command_name)
  end
end

#say(*args) ⇒ Object

:nodoc:



473
474
475
# File 'lib/simple_commander/runner.rb', line 473

def say(*args) #:nodoc:
  $terminal.say(*args)
end

#valid_command_names_from(*args) ⇒ Object

Returns array of valid command names found within args.



272
273
274
275
# File 'lib/simple_commander/runner.rb', line 272

def valid_command_names_from(*args)
  arg_string = args.delete_if { |value| value =~ /^-/ }.join ' '
  commands.keys.find_all { |name| name if /^#{name}\b/.match arg_string }
end

#versionObject

Return program version.



111
112
113
# File 'lib/simple_commander/runner.rb', line 111

def version
  format('%s %s', program(:name), program(:version))
end